 #ifndef __FIS_H__
#define __FIS_H__

/***********************************************************************
 Macros and definitions
 **********************************************************************/

#define ABS(x)   ( (x) > (0) ? (x): (-(x)) )
#define MAX(x,y) ( (x) > (y) ? (x) : (y) )
#define MIN(x,y) ( (x) < (y) ? (x) : (y) )
#define MF_PARA_N 4
#define STR_LEN 500
#define MF_POINT_N 101
/* for debugging */
#ifdef __STDC__
#define PRINT(expr) printf(#expr " = %g\n", (double)expr)
#define PRINTMAT(mat,m,n) printf(#mat " = \n"); fisPrintMatrix(mat,m,n)
#else
#define PRINT(expr) printf("%g\n", (double)expr)
#define PRINTMAT(mat,m,n) fisPrintMatrix(mat,m,n)
#endif

#define FREEMAT(mat,m) fisFreeMatrix(mat,m)
#define FREEARRAY(array) free(array)
/*
#define FREEMAT(mat,m) printf("Free " #mat " ...\n"); fisFreeMatrix(mat,m)
#define FREEARRAY(array) printf("Free " #array " ...\n"); free(array)
*/

/***********************************************************************
 Data types
 **********************************************************************/

typedef struct fis_node {
	int handle;
	int load_param;
	char name[STR_LEN];
	char type[STR_LEN];
	char andMethod[STR_LEN];
	char orMethod[STR_LEN];
	char impMethod[STR_LEN];
	char aggMethod[STR_LEN];
	char defuzzMethod[STR_LEN];
	int in_n;
	int out_n;
	int rule_n;
	int **rule_list;
	double *rule_weight;
	int *and_or;	/* AND-OR indicator */
	double *firing_strength;
	double *rule_output;
	/* Sugeno: output for each rules */
	/* Mamdani: constrained output MF values of rules */
	struct io_node **input;
	struct io_node **output;
#ifdef __STDC__
	double (*andFcn)(double, double);
	double (*orFcn)(double, double);
	double (*impFcn)(double, double);
	double (*aggFcn)(double, double);
#else
	double (*andFcn)();
	double (*orFcn)();
	double (*impFcn)();
	double (*aggFcn)();
#endif
	double (*defuzzFcn)();
	double *fcn_value; /* used in defuzzify */
	double *mfs_of_rule;	/* MF values in a rule */
	struct fis_node *next;
} FIS;

typedef struct io_node {
	char name[STR_LEN];
	int mf_n;
	double bound[2];
	double value;
	struct mf_node **mf;
} IO;

typedef struct mf_node {
	char label[STR_LEN];
	char type[STR_LEN];
#ifdef __STDC__
	double (*mfFcn)(double, double *); /* pointer to a mem. fcn */ 
#else
	double (*mfFcn)(); /* pointer to a mem. fcn */ 
#endif
	double para[MF_PARA_N];
	double *sugeno_coef;	/* for Sugeno only */
	double value;		/* for Sugeno only */
	double *value_array;	/* for Mamdani only, array of MF values */
} MF;

void fisBuildFisNode(FIS *fis, double **fismatrix, int col_n);
double ** returnFismatrix(char *fis_file, int *row_n_p, int *col_n_p);
void fisFreeMatrix(void **matrix, int row_n);
void fisFreeFisNode(FIS *fis);
/*
double EvalRules(double **fisMatrix, int fis_row_n, int fis_col_n,
                 double *dataMatrix, int data_row_n, int data_col_n,
                 FIS *fis);
*/
typedef struct {
  double **fisMatrix;
  int fis_row_n, fis_col_n;
  int data_row_n, data_col_n;
  FIS *fis;
} FIS_STUFF;


static char FISPATH[100]={"/u3/fuzzy/fis/"};

int InitializeFIS(FIS_STUFF *Fiss, int TotalFisFiles,
                  char *FileName, int FileNameSize);
void FreeFIS(FIS_STUFF *Fiss, int TotalFisFiles);
void SetFisPathDir(char *FisPathDir);

#endif

