#include <stdio.h>
#include <stdlib.h>
#include <float.h>

#include "fis.h"

#include "fis.c"

static double EvalRules(double **fisMatrix, int fis_row_n, int fis_col_n,
                        double *dataMatrix, int data_row_n, int data_col_n,
                        FIS *fis);
#define MAXINPUTS 10
/***********************************************************************
 main routine 
 **********************************************************************/
double FuzzyGraph(char *FisFile, int NInputs, double *Inputs)
{
  int i;
  double **fisMatrix;
  int fis_row_n, fis_col_n; 
  int data_row_n, data_col_n;
  FIS *fis;
  double Value;
  double dataMatrix[MAXINPUTS];

    if (NInputs>=MAXINPUTS) return(DBL_MIN);

    fisMatrix = returnFismatrix(FisFile, &fis_row_n, &fis_col_n);

    /* obtain data matrix and FIS matrix */
    data_row_n=1; /* assumes only one output */
    data_col_n=NInputs;
    for(i=0;i<NInputs;i++) dataMatrix[i]=Inputs[i];

    /* build FIS data structure */
    fis = (FIS *)calloc(1, sizeof(FIS));
    fisBuildFisNode(fis, fisMatrix, fis_col_n);

    Value=EvalRules(fisMatrix,fis_row_n,fis_col_n,
                    dataMatrix,data_row_n,data_col_n,fis);

  return(Value);
}
/***********************************************************************
 Eval routine 
 **********************************************************************/
static double EvalRules(double **fisMatrix, int fis_row_n, int fis_col_n,
                        double *dataMatrix, int data_row_n, int data_col_n,
                        FIS *fis)
{
  int i, j;
  int debug = 0;
  double OutputEvaluation[1][1];
  double OE;


	/* error checking */
	if (data_col_n < fis->in_n) {
		printf("Given FIS is a %d-input %d-output system.\n",
			fis->in_n, fis->out_n);
		printf("Given data file does not have enough input entries.\n");
		fisError("Exiting ...");
	}

	/* debugging */
	if (debug) fisPrintData(fis);

	/* evaluate FIS on each input vector */
	getFisOutput(dataMatrix, fis, OutputEvaluation[0]);

        OE=OutputEvaluation[0][0];

   return(OE);
}
